#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
from unittest import mock

import webob.exc

import heat.api.middleware.fault as fault
import heat.api.openstack.v1.actions as actions
from heat.common import identifier
from heat.common import policy
from heat.rpc import client as rpc_client
from heat.tests.api.openstack_v1 import tools
from heat.tests import common


@mock.patch.object(policy.Enforcer, 'enforce')
class ActionControllerTest(tools.ControllerTest, common.HeatTestCase):
    """Tests the API class ActionController.

    Tests the API class which acts as the WSGI controller,
    the endpoint processing API requests after they are routed
    """

    def setUp(self):
        super(ActionControllerTest, self).setUp()
        # Create WSGI controller instance

        class DummyConfig(object):
            bind_port = 8004

        cfgopts = DummyConfig()
        self.controller = actions.ActionController(options=cfgopts)

    def test_action_suspend(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'suspend', True)
        stack_identity = identifier.HeatIdentifier(self.tenant,
                                                   'wordpress', '1')
        body = {'suspend': None}
        req = self._post(stack_identity._tenant_path() + '/actions',
                         data=json.dumps(body))

        mock_call = self.patchobject(rpc_client.EngineClient, 'call',
                                     return_value=None)

        result = self.controller.action(req, tenant_id=self.tenant,
                                        stack_name=stack_identity.stack_name,
                                        stack_id=stack_identity.stack_id,
                                        body=body)
        self.assertIsNone(result)

        mock_call.assert_called_once_with(
            req.context,
            ('stack_suspend', {'stack_identity': stack_identity})
        )

    def test_action_resume(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'resume', True)
        stack_identity = identifier.HeatIdentifier(self.tenant,
                                                   'wordpress', '1')
        body = {'resume': None}
        req = self._post(stack_identity._tenant_path() + '/actions',
                         data=json.dumps(body))

        mock_call = self.patchobject(rpc_client.EngineClient, 'call',
                                     return_value=None)

        result = self.controller.action(req, tenant_id=self.tenant,
                                        stack_name=stack_identity.stack_name,
                                        stack_id=stack_identity.stack_id,
                                        body=body)
        self.assertIsNone(result)

        mock_call.assert_called_once_with(
            req.context,
            ('stack_resume', {'stack_identity': stack_identity})
        )

    def test_action_check(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'check', True)
        stack_identity = identifier.HeatIdentifier(self.tenant,
                                                   'wordpress', '1')
        body = {'check': None}
        req = self._post(stack_identity._tenant_path() + '/actions',
                         data=json.dumps(body))

        mock_call = self.patchobject(rpc_client.EngineClient, 'call',
                                     return_value=None)

        result = self.controller.action(req, tenant_id=self.tenant,
                                        stack_name=stack_identity.stack_name,
                                        stack_id=stack_identity.stack_id,
                                        body=body)
        self.assertIsNone(result)

        mock_call.assert_called_once_with(
            req.context,
            ('stack_check', {'stack_identity': stack_identity})
        )

    def _test_action_cancel_update(self, mock_enforce, with_rollback=True):
        act = 'cancel_update' if with_rollback else 'cancel_without_rollback'
        self._mock_enforce_setup(mock_enforce, act, True)
        stack_identity = identifier.HeatIdentifier(self.tenant,
                                                   'wordpress', '1')
        body = {act: None}
        req = self._post(stack_identity._tenant_path() + '/actions',
                         data=json.dumps(body))

        client = self.patchobject(rpc_client.EngineClient, 'call')
        result = self.controller.action(req, tenant_id=self.tenant,
                                        stack_name=stack_identity.stack_name,
                                        stack_id=stack_identity.stack_id,
                                        body=body)
        self.assertIsNone(result)
        client.assert_called_with(
            req.context,
            ('stack_cancel_update',
             {'stack_identity': stack_identity,
              'cancel_with_rollback': with_rollback}),
            version='1.14')

    def test_action_cancel_update(self, mock_enforce):
        self._test_action_cancel_update(mock_enforce)

    def test_action_cancel_without_rollback(self, mock_enforce):
        self._test_action_cancel_update(mock_enforce, False)

    def test_action_badaction(self, mock_enforce):
        stack_identity = identifier.HeatIdentifier(self.tenant,
                                                   'wordpress', '1')
        body = {'notallowed': None}
        req = self._post(stack_identity._tenant_path() + '/actions',
                         data=json.dumps(body))

        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.action,
                          req, tenant_id=self.tenant,
                          stack_name=stack_identity.stack_name,
                          stack_id=stack_identity.stack_id,
                          body=body)

    def test_action_badaction_empty(self, mock_enforce):
        stack_identity = identifier.HeatIdentifier(self.tenant,
                                                   'wordpress', '1')
        body = {}
        req = self._post(stack_identity._tenant_path() + '/actions',
                         data=json.dumps(body))

        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.action,
                          req, tenant_id=self.tenant,
                          stack_name=stack_identity.stack_name,
                          stack_id=stack_identity.stack_id,
                          body=body)

    def test_action_badaction_multiple(self, mock_enforce):
        stack_identity = identifier.HeatIdentifier(self.tenant,
                                                   'wordpress', '1')
        body = {'one': None, 'two': None}
        req = self._post(stack_identity._tenant_path() + '/actions',
                         data=json.dumps(body))

        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.action,
                          req, tenant_id=self.tenant,
                          stack_name=stack_identity.stack_name,
                          stack_id=stack_identity.stack_id,
                          body=body)

    def test_action_rmt_aterr(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'suspend', True)
        stack_identity = identifier.HeatIdentifier(self.tenant,
                                                   'wordpress', '1')
        body = {'suspend': None}
        req = self._post(stack_identity._tenant_path() + '/actions',
                         data=json.dumps(body))

        mock_call = self.patchobject(
            rpc_client.EngineClient, 'call',
            side_effect=tools.to_remote_error(AttributeError()))

        resp = tools.request_with_middleware(
            fault.FaultWrapper,
            self.controller.action,
            req, tenant_id=self.tenant,
            stack_name=stack_identity.stack_name,
            stack_id=stack_identity.stack_id,
            body=body)

        self.assertEqual(400, resp.json['code'])
        self.assertEqual('AttributeError', resp.json['error']['type'])

        mock_call.assert_called_once_with(
            req.context,
            ('stack_suspend', {'stack_identity': stack_identity})
        )

    def test_action_err_denied_policy(self, mock_enforce):
        self._mock_enforce_setup(mock_enforce, 'suspend', False)
        stack_identity = identifier.HeatIdentifier(self.tenant,
                                                   'wordpress', '1')
        body = {'suspend': None}
        req = self._post(stack_identity._tenant_path() + '/actions',
                         data=json.dumps(body))

        resp = tools.request_with_middleware(
            fault.FaultWrapper,
            self.controller.action,
            req, tenant_id=self.tenant,
            stack_name=stack_identity.stack_name,
            stack_id=stack_identity.stack_id,
            body=body)
        self.assertEqual(403, resp.status_int)
        self.assertIn('403 Forbidden', str(resp))

    def test_action_badaction_ise(self, mock_enforce):
        stack_identity = identifier.HeatIdentifier(self.tenant,
                                                   'wordpress', '1')
        body = {'oops': None}
        req = self._post(stack_identity._tenant_path() + '/actions',
                         data=json.dumps(body))

        self.controller.ACTIONS = (SUSPEND, NEW) = ('suspend', 'oops')

        self.assertRaises(webob.exc.HTTPInternalServerError,
                          self.controller.action,
                          req, tenant_id=self.tenant,
                          stack_name=stack_identity.stack_name,
                          stack_id=stack_identity.stack_id,
                          body=body)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock
import uuid

from glanceclient import exc

from heat.engine.clients import client_exception as exception
from heat.engine.clients.os import glance
from heat.tests import common
from heat.tests import utils


class GlanceUtilsTest(common.HeatTestCase):
    """Basic tests for :module:'heat.engine.resources.clients.os.glance'."""

    def setUp(self):
        super(GlanceUtilsTest, self).setUp()
        self.glance_client = mock.MagicMock()
        con = utils.dummy_context()
        c = con.clients
        self.glance_plugin = c.client_plugin('glance')
        self.glance_plugin.client = lambda: self.glance_client
        self.my_image = mock.MagicMock()

    def test_find_image_by_name_or_id(self):
        """Tests the find_image_by_name_or_id function."""
        img_id = str(uuid.uuid4())
        img_name = 'myfakeimage'
        self.my_image.id = img_id
        self.my_image.name = img_name
        self.glance_client.images.get.side_effect = [
            self.my_image,
            exc.HTTPNotFound(),
            exc.HTTPNotFound(),
            exc.HTTPNotFound()]
        self.glance_client.images.list.side_effect = [
            [self.my_image],
            [],
            [self.my_image, self.my_image]]
        self.assertEqual(img_id,
                         self.glance_plugin.find_image_by_name_or_id(img_id))
        self.assertEqual(img_id,
                         self.glance_plugin.find_image_by_name_or_id(img_name))
        self.assertRaises(exception.EntityMatchNotFound,
                          self.glance_plugin.find_image_by_name_or_id,
                          'noimage')
        self.assertRaises(exception.EntityUniqueMatchNotFound,
                          self.glance_plugin.find_image_by_name_or_id,
                          'myfakeimage')


class ImageConstraintTest(common.HeatTestCase):

    def setUp(self):
        super(ImageConstraintTest, self).setUp()
        self.ctx = utils.dummy_context()
        self.mock_find_image = mock.Mock()
        self.ctx.clients.client_plugin(
            'glance').find_image_by_name_or_id = self.mock_find_image
        self.constraint = glance.ImageConstraint()

    def test_validation(self):
        self.mock_find_image.side_effect = [
            "id1", exception.EntityMatchNotFound(),
            exception.EntityUniqueMatchNotFound()]
        self.assertTrue(self.constraint.validate("foo", self.ctx))
        self.assertFalse(self.constraint.validate("bar", self.ctx))
        self.assertFalse(self.constraint.validate("baz", self.ctx))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_policy import policy

RULE_CONTEXT_IS_ADMIN = 'rule:context_is_admin'
RULE_PROJECT_ADMIN = 'rule:project_admin'
RULE_DENY_STACK_USER = 'rule:deny_stack_user'
RULE_DENY_EVERYBODY = 'rule:deny_everybody'
RULE_ALLOW_EVERYBODY = 'rule:allow_everybody'

# Check strings that embody common personas
SYSTEM_ADMIN = 'role:admin and system_scope:all'
SYSTEM_READER = 'role:reader and system_scope:all'
PROJECT_MEMBER = 'role:member and project_id:%(project_id)s'
PROJECT_READER = 'role:reader and project_id:%(project_id)s'

# Heat personas
PROJECT_ADMIN = 'role:admin and project_id:%(project_id)s'
PROJECT_STACK_USER = 'role:heat_stack_user and project_id:%(project_id)s'

# Composite check strings that are useful for policies that protect APIs that
# operate at different scopes.
PROJECT_MEMBER_OR_STACK_USER = (
    '(' + PROJECT_MEMBER + ')'
    ' or (' + PROJECT_STACK_USER + ')'
)
PROJECT_READER_OR_STACK_USER = (
    '(' + PROJECT_READER + ')'
    ' or (' + PROJECT_STACK_USER + ')'
)


rules = [
    policy.RuleDefault(
        name="context_is_admin",
        check_str=(
            "(role:admin and is_admin_project:True) OR "
            "(" + SYSTEM_ADMIN + ")"
        ),
        description="Decides what is required for the 'is_admin:True' check "
        "to succeed."),
    policy.RuleDefault(
        name="project_admin",
        check_str="role:admin",
        description="Default rule for project admin."),
    policy.RuleDefault(
        name="deny_stack_user",
        check_str="not role:heat_stack_user",
        description="Default rule for deny stack user."),
    policy.RuleDefault(
        name="deny_everybody",
        check_str="!",
        description="Default rule for deny everybody."),
    policy.RuleDefault(
        name="allow_everybody",
        check_str="",
        description="Default rule for allow everybody.")
]


def list_rules():
    return rules



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from heat.common import exception
from heat.common import template_format
from heat.engine.resources.openstack.neutron import security_group_rule
from heat.tests import common
from heat.tests.openstack.neutron import inline_templates
from heat.tests import utils


class SecurityGroupRuleTest(common.HeatTestCase):

    def test_resource_mapping(self):
        mapping = security_group_rule.resource_mapping()
        self.assertEqual(mapping['OS::Neutron::SecurityGroupRule'],
                         security_group_rule.SecurityGroupRule)

    @mock.patch('heat.engine.clients.os.neutron.'
                'NeutronClientPlugin.has_extension', return_value=True)
    def _create_stack(self, ext_func,
                      tmpl=inline_templates.SECURITY_GROUP_RULE_TEMPLATE):
        self.t = template_format.parse(tmpl)
        self.stack = utils.parse_stack(self.t)
        self.sg_rule = self.stack['security_group_rule']
        self.neutron_client = mock.MagicMock()
        self.sg_rule.client = mock.MagicMock(return_value=self.neutron_client)

        self.sg_rule.client_plugin().find_resourceid_by_name_or_id = (
            mock.MagicMock(return_value='123'))

    def test_create(self):
        self._create_stack()
        self.neutron_client.create_security_group_rule.return_value = {
            'security_group_rule': {'id': '1234'}}
        expected = {
            'security_group_rule': {
                'security_group_id': '123',
                'description': 'test description',
                'remote_group_id': '123',
                'protocol': 'tcp',
                'port_range_min': '100',
                'direction': 'ingress',
                'ethertype': 'IPv4'
            }
        }

        self.sg_rule.handle_create()

        self.neutron_client.create_security_group_rule.assert_called_with(
            expected)

    def test_validate_conflict_props(self):
        self.patchobject(security_group_rule.SecurityGroupRule,
                         'is_service_available',
                         return_value=(True, None))

        tmpl = inline_templates.SECURITY_GROUP_RULE_TEMPLATE
        tmpl += '      remote_ip_prefix: "10.0.0.0/8"'
        self._create_stack(tmpl=tmpl)

        self.assertRaises(exception.ResourcePropertyConflict,
                          self.sg_rule.validate)

    def test_validate_max_port_less_than_min_port(self):
        self.patchobject(security_group_rule.SecurityGroupRule,
                         'is_service_available',
                         return_value=(True, None))

        tmpl = inline_templates.SECURITY_GROUP_RULE_TEMPLATE
        tmpl += '      port_range_max: 50'
        self._create_stack(tmpl=tmpl)

        self.assertRaises(exception.StackValidationFailed,
                          self.sg_rule.validate)

    def test_show_resource(self):
        self._create_stack()
        self.sg_rule.resource_id_set('1234')
        self.neutron_client.show_security_group_rule.return_value = {
            'security_group_rule': {'id': '1234'}
        }

        self.assertEqual({'id': '1234'}, self.sg_rule._show_resource())
        self.neutron_client.show_security_group_rule.assert_called_with('1234')

    def test_delete(self):
        self._create_stack()
        self.sg_rule.resource_id_set('1234')

        self.sg_rule.handle_delete()

        self.neutron_client.delete_security_group_rule.assert_called_with(
            '1234')



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import collections

from oslo_config import cfg
from oslo_serialization import jsonutils
import yaml

from heat.common import exception
from heat.common.i18n import _

if hasattr(yaml, 'CSafeLoader'):
    _yaml_loader_base = yaml.CSafeLoader
else:
    _yaml_loader_base = yaml.SafeLoader


class yaml_loader(_yaml_loader_base):
    def _construct_yaml_str(self, node):
        # Override the default string handling function
        # to always return unicode objects
        return self.construct_scalar(node)


if hasattr(yaml, 'CSafeDumper'):
    _yaml_dumper_base = yaml.CSafeDumper
else:
    _yaml_dumper_base = yaml.SafeDumper


class yaml_dumper(_yaml_dumper_base):
    def represent_ordered_dict(self, data):
        return self.represent_dict(data.items())


yaml_loader.add_constructor('tag:yaml.org,2002:str',
                            yaml_loader._construct_yaml_str)
# Unquoted dates like 2013-05-23 in yaml files get loaded as objects of type
# datetime.data which causes problems in API layer when being processed by
# openstack.common.jsonutils. Therefore, make unicode string out of timestamps
# until jsonutils can handle dates.
yaml_loader.add_constructor('tag:yaml.org,2002:timestamp',
                            yaml_loader._construct_yaml_str)

yaml_dumper.add_representer(collections.OrderedDict,
                            yaml_dumper.represent_ordered_dict)


def simple_parse(tmpl_str, tmpl_url=None):
    try:
        tpl = jsonutils.loads(tmpl_str)
    except ValueError:
        try:
            tpl = yaml.load(tmpl_str, Loader=yaml_loader)
        except yaml.YAMLError:
            # NOTE(prazumovsky): we need to return more informative error for
            # user, so use SafeLoader, which return error message with template
            # snippet where error has been occurred.
            try:
                tpl = yaml.load(tmpl_str, Loader=yaml.SafeLoader)
            except yaml.YAMLError as yea:
                if tmpl_url is None:
                    tmpl_url = '[root stack]'
                yea = str(yea)
                msg = _('Error parsing template %(tmpl)s '
                        '%(yea)s') % {'tmpl': tmpl_url, 'yea': yea}
                raise ValueError(msg)
        else:
            if tpl is None:
                tpl = {}

    if not isinstance(tpl, dict):
        raise ValueError(_('The template is not a JSON object '
                           'or YAML mapping.'))

    return tpl


def validate_template_limit(contain_str):
    """Validate limit for the template.

    Check if the contain exceeds allowed size range.
    """

    if len(contain_str) > cfg.CONF.max_template_size:
        msg = _("Template size (%(actual_len)s bytes) exceeds maximum "
                "allowed size (%(limit)s bytes)."
                ) % {'actual_len': len(contain_str),
                     'limit': cfg.CONF.max_template_size}
        raise exception.RequestLimitExceeded(message=msg)


def parse(tmpl_str, tmpl_url=None):
    """Takes a string and returns a dict containing the parsed structure.

    This includes determination of whether the string is using the
    JSON or YAML format.
    """

    # TODO(ricolin): Move this validation to api side.
    # Validate nested stack template.
    validate_template_limit(str(tmpl_str))

    tpl = simple_parse(tmpl_str, tmpl_url)
    # Looking for supported version keys in the loaded template
    if not ('HeatTemplateFormatVersion' in tpl
            or 'heat_template_version' in tpl
            or 'AWSTemplateFormatVersion' in tpl):
        raise ValueError(_("Template format version not found."))
    return tpl


def convert_json_to_yaml(json_str):
    """Convert AWS JSON template format to Heat YAML format.

    :param json_str: a string containing the AWS JSON template format.
    :returns: the equivalent string containing the Heat YAML format.
    """

    # parse the string as json to a python structure
    tpl = jsonutils.loads(json_str, object_pairs_hook=collections.OrderedDict)

    # Replace AWS format version with Heat format version
    def top_level_items(tpl):
        yield ("HeatTemplateFormatVersion", '2012-12-12')

        for k, v in tpl.items():
            if k != 'AWSTemplateFormatVersion':
                yield k, v

    # dump python structure to yaml
    return yaml.dump(collections.OrderedDict(top_level_items(tpl)),
                     Dumper=yaml_dumper)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from heat.engine import api as engine_api
from heat.engine import notification


def send(stack):
    """Send usage notifications to the configured notification driver."""

    # The current notifications have a start/end:
    # see: https://wiki.openstack.org/wiki/SystemUsageData
    # so to be consistent we translate our status into a known start/end/error
    # suffix.
    level = notification.get_default_level()
    if stack.status == stack.IN_PROGRESS:
        suffix = 'start'
    elif stack.status == stack.COMPLETE:
        suffix = 'end'
    else:
        suffix = 'error'
        level = notification.ERROR

    event_type = '%s.%s.%s' % ('stack',
                               stack.action.lower(),
                               suffix)

    notification.notify(stack.context, event_type, level,
                        engine_api.format_notification_body(stack))



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid

from heat.common import short_id
from heat.tests import common


class ShortIdTest(common.HeatTestCase):

    def test_byte_string_8(self):
        self.assertEqual(b'\xab', short_id._to_byte_string(0xab, 8))
        self.assertEqual(b'\x05', short_id._to_byte_string(0x05, 8))

    def test_byte_string_16(self):
        self.assertEqual(b'\xab\xcd', short_id._to_byte_string(0xabcd, 16))
        self.assertEqual(b'\x0a\xbc', short_id._to_byte_string(0xabc, 16))

    def test_byte_string_12(self):
        self.assertEqual(b'\xab\xc0', short_id._to_byte_string(0xabc, 12))
        self.assertEqual(b'\x0a\xb0', short_id._to_byte_string(0x0ab, 12))

    def test_byte_string_60(self):
        val = 0x111111111111111
        byte_string = short_id._to_byte_string(val, 60)
        self.assertEqual(b'\x11\x11\x11\x11\x11\x11\x11\x10', byte_string)

    def test_get_id_string(self):
        id = short_id.get_id('11111111-1111-4111-bfff-ffffffffffff')
        self.assertEqual('ceirceirceir', id)

    def test_get_id_uuid_1(self):
        source = uuid.UUID('11111111-1111-4111-bfff-ffffffffffff')
        self.assertEqual(0x111111111111111, source.time)
        self.assertEqual('ceirceirceir', short_id.get_id(source))

    def test_get_id_uuid_f(self):
        source = uuid.UUID('ffffffff-ffff-4fff-8000-000000000000')
        self.assertEqual('777777777777', short_id.get_id(source))

    def test_get_id_uuid_0(self):
        source = uuid.UUID('00000000-0000-4000-bfff-ffffffffffff')
        self.assertEqual('aaaaaaaaaaaa', short_id.get_id(source))

    def test_get_id_uuid_endianness(self):
        source = uuid.UUID('ffffffff-00ff-4000-aaaa-aaaaaaaaaaaa')
        self.assertEqual('aaaa77777777', short_id.get_id(source))

    def test_get_id_uuid1(self):
        source = uuid.uuid1()
        self.assertRaises(ValueError, short_id.get_id, source)

    def test_generate_ids(self):
        allowed_chars = [ord(c) for c in 'abcdefghijklmnopqrstuvwxyz234567']
        ids = [short_id.generate_id() for i in range(25)]

        for id in ids:
            self.assertEqual(12, len(id))
            self.assertFalse(id.translate({c: None for c in allowed_chars}))
            self.assertEqual(1, ids.count(id))



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from heat.common import exception
from heat.common.i18n import _
from heat.engine import attributes
from heat.engine import constraints
from heat.engine import properties
from heat.engine.resources.openstack.octavia import octavia_base
from heat.engine import translation


class L7Policy(octavia_base.OctaviaBase):
    """A resource for managing octavia L7Policies.

    This resource manages L7Policies, which represent a collection of L7Rules.
    L7Policy holds the action that should be performed when the rules are
    matched (Redirect to Pool, Redirect to URL, Reject). L7Policy holds a
    Listener id, so a Listener can evaluate a collection of L7Policies.
    L7Policy will return True when all of the L7Rules that belong
    to this L7Policy are matched. L7Policies under a specific Listener are
    ordered and the first l7Policy that returns a match will be executed.
    When none of the policies match the request gets forwarded to
    listener.default_pool_id.
    """

    PROPERTIES = (
        NAME, DESCRIPTION, ADMIN_STATE_UP, ACTION,
        REDIRECT_POOL, REDIRECT_URL, POSITION, LISTENER
    ) = (
        'name', 'description', 'admin_state_up', 'action',
        'redirect_pool', 'redirect_url', 'position', 'listener'
    )

    L7ACTIONS = (
        REJECT, REDIRECT_TO_POOL, REDIRECT_TO_URL
    ) = (
        'REJECT', 'REDIRECT_TO_POOL', 'REDIRECT_TO_URL'
    )

    ATTRIBUTES = (RULES_ATTR) = ('rules')

    properties_schema = {
        NAME: properties.Schema(
            properties.Schema.STRING,
            _('Name of the policy.'),
            update_allowed=True
        ),
        DESCRIPTION: properties.Schema(
            properties.Schema.STRING,
            _('Description of the policy.'),
            update_allowed=True
        ),
        ADMIN_STATE_UP: properties.Schema(
            properties.Schema.BOOLEAN,
            _('The administrative state of the policy.'),
            default=True,
            update_allowed=True
        ),
        ACTION: properties.Schema(
            properties.Schema.STRING,
            _('Action type of the policy.'),
            required=True,
            constraints=[constraints.AllowedValues(L7ACTIONS)],
            update_allowed=True
        ),
        REDIRECT_POOL: properties.Schema(
            properties.Schema.STRING,
            _('ID or name of the pool for REDIRECT_TO_POOL action type.'),
            constraints=[
                constraints.CustomConstraint('octavia.pool')
            ],
            update_allowed=True
        ),
        REDIRECT_URL: properties.Schema(
            properties.Schema.STRING,
            _('URL for REDIRECT_TO_URL action type. '
              'This should be a valid URL string.'),
            update_allowed=True
        ),
        POSITION: properties.Schema(
            properties.Schema.NUMBER,
            _('L7 policy position in ordered policies list. This must be '
              'an integer starting from 1. If not specified, policy will be '
              'placed at the tail of existing policies list.'),
            constraints=[constraints.Range(min=1)],
            update_allowed=True
        ),
        LISTENER: properties.Schema(
            properties.Schema.STRING,
            _('ID or name of the listener this policy belongs to.'),
            required=True,
            constraints=[
                constraints.CustomConstraint('octavia.listener')
            ]
        ),
    }

    attributes_schema = {
        RULES_ATTR: attributes.Schema(
            _('L7Rules associated with this policy.'),
            type=attributes.Schema.LIST
        ),
    }

    def translation_rules(self, props):
        return [
            translation.TranslationRule(
                props,
                translation.TranslationRule.RESOLVE,
                [self.LISTENER],
                client_plugin=self.client_plugin(),
                finder='get_listener',
            ),
            translation.TranslationRule(
                props,
                translation.TranslationRule.RESOLVE,
                [self.REDIRECT_POOL],
                client_plugin=self.client_plugin(),
                finder='get_pool',
            ),
        ]

    def validate(self):
        super(L7Policy, self).validate()
        if (self.properties[self.ACTION] == self.REJECT and
                (self.properties[self.REDIRECT_POOL] is not None or
                 self.properties[self.REDIRECT_URL] is not None)):
            msg = (_('Properties %(pool)s and %(url)s are not required when '
                     '%(action)s type is set to %(action_type)s.') %
                   {'pool': self.REDIRECT_POOL,
                    'url': self.REDIRECT_URL,
                    'action': self.ACTION,
                    'action_type': self.REJECT})
            raise exception.StackValidationFailed(message=msg)

        if self.properties[self.ACTION] == self.REDIRECT_TO_POOL:
            if self.properties[self.REDIRECT_URL] is not None:
                raise exception.ResourcePropertyValueDependency(
                    prop1=self.REDIRECT_URL,
                    prop2=self.ACTION,
                    value=self.REDIRECT_TO_URL)
            if self.properties[self.REDIRECT_POOL] is None:
                msg = (_('Property %(pool)s is required when %(action)s '
                         'type is set to %(action_type)s.') %
                       {'pool': self.REDIRECT_POOL,
                        'action': self.ACTION,
                        'action_type': self.REDIRECT_TO_POOL})
                raise exception.StackValidationFailed(message=msg)

        if self.properties[self.ACTION] == self.REDIRECT_TO_URL:
            if self.properties[self.REDIRECT_POOL] is not None:
                raise exception.ResourcePropertyValueDependency(
                    prop1=self.REDIRECT_POOL,
                    prop2=self.ACTION,
                    value=self.REDIRECT_TO_POOL)
            if self.properties[self.REDIRECT_URL] is None:
                msg = (_('Property %(url)s is required when %(action)s '
                         'type is set to %(action_type)s.') %
                       {'url': self.REDIRECT_URL,
                        'action': self.ACTION,
                        'action_type': self.REDIRECT_TO_URL})
                raise exception.StackValidationFailed(message=msg)

    def _prepare_args(self, properties):
        props = dict((k, v) for k, v in properties.items()
                     if v is not None)
        if self.NAME not in props:
            props[self.NAME] = self.physical_resource_name()
        props['listener_id'] = props.pop(self.LISTENER)
        if self.REDIRECT_POOL in props:
            props['redirect_pool_id'] = props.pop(self.REDIRECT_POOL)
        return props

    def _resource_create(self, properties):
        return self.client().l7policy_create(
            json={'l7policy': properties})['l7policy']

    def _resource_update(self, prop_diff):
        if self.REDIRECT_POOL in prop_diff:
            prop_diff['redirect_pool_id'] = prop_diff.pop(self.REDIRECT_POOL)
        self.client().l7policy_set(
            self.resource_id, json={'l7policy': prop_diff})

    def _resource_delete(self):
        self.client().l7policy_delete(self.resource_id)

    def _show_resource(self):
        return self.client().l7policy_show(self.resource_id)


def resource_mapping():
    return {
        'OS::Octavia::L7Policy': L7Policy
    }
